unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, TypInfo, Grids, Mask, ComCtrls, Db, DBTables,
  DBCtrls, DBGrids, Buttons, DRIntf,
  dnDBPxG, dnPxMask, dnPxProp, dnPxFlds, dnPxTbl, dnGlobal, dnPxGrid, dnUtil,
  dnDBPxM;

type
  TForm1 = class(TForm)
    StatusBar1: TStatusBar;
    DataSource1: TDataSource;
    PxTable1: TPxTable;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    PxComboBox1: TPxComboBox;
    PxMaskEdit1: TPxMaskEdit;
    PxGrid1: TPxGrid;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    cbxAutoFill: TCheckBox;
    cbxAutoValidate: TCheckBox;
    cbxRaiseInputError: TCheckBox;
    Button2: TButton;
    DBPxComboBox1: TDBPxComboBox;
    DBPxGrid1: TDBPxGrid;
    DBNavigator1: TDBNavigator;
    DBPxEdit1: TDBPxEdit;
    Label3: TLabel;
    Label4: TLabel;
    lblPxMask: TEdit;
    lblErrMsg: TEdit;
    PxTable1VendorNo: TPxFloatField;
    PxTable1VendorName: TPxStringField;
    PxTable1Address1: TPxStringField;
    PxTable1Address2: TPxStringField;
    PxTable1City: TPxStringField;
    PxTable1State: TPxStringField;
    PxTable1Zip: TPxStringField;
    PxTable1Country: TPxStringField;
    PxTable1Phone: TPxStringField;
    PxTable1FAX: TPxStringField;
    PxTable1Preferred: TBooleanField;
    procedure PxGrid1GetEditMask(Sender: TObject; ACol, ARow: Integer;
      var Value: string);
    procedure PxMaskEdit1ValidChanged(Sender: TControlPxMask;
      EditControl: TWinControl; TextValid: Boolean);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PageControl1Changing(Sender: TObject;
      var AllowChange: Boolean);
    procedure Button2Click(Sender: TObject);
    procedure PxTable1AfterOpen(DataSet: TDataSet);
    procedure FormCreate(Sender: TObject);
    procedure DBPxGrid1ColEnter(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure TabSheet1DragDrop(Sender, Source: TObject; X, Y: Integer);
  private
    { Private declarations }
    LastMask: TWinControl;
    procedure SetPxMaskValues(Mask: TPxMask);
    procedure AppEx(Sender: TObject; E: Exception);
    procedure ActiveControlChanged(Sender: TObject);
  public
    { Public declarations }
    constructor create(aOwner: TComponent); override;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Consts;

procedure TForm1.AppEx(Sender: TObject; E: Exception);
begin
  StatusBar1.SimpleText := E.Message;
end;

constructor TForm1.create(aOwner: TComponent);
begin
  Application.OnException := AppEx;
  Screen.OnActiveControlChange := ActiveControlChanged;
  inherited;
end;

procedure TForm1.PxGrid1GetEditMask(Sender: TObject; ACol, ARow: Integer;
  var Value: string);
begin
  with TPxGrid(Sender).PxMask do
    case ACol of
      1:
        begin
          Mask := '{Yes,No}';
          RaiseInputError := true;
        end;
      2: Mask := '*5{#}[-*4#]';
      3: Mask := '*[&[*?]*[{#, ,.,;,,;;,:,;[,;],(,),}]]';
      4: Mask := '{Red,Gr{ay,een},B{l{ack,ue},rown},White,Yellow}';
    else
      Mask := '';
      Value := '!90:00:00;1;_';
    end;
  ActiveControlChanged(Sender);
end;

procedure TForm1.PxMaskEdit1ValidChanged(Sender: TControlPxMask;
  EditControl: TWinControl; TextValid: Boolean);
begin
  if TextValid then
    TForm1(EditControl).Font.Color := clBlack
  else
    TForm1(EditControl).Font.Color := clRed;
end;

procedure TForm1.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  StatusBar1.SimpleText := '';
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  StatusBar1.SimpleText := '';
end;

procedure TForm1.PageControl1Changing(Sender: TObject;
  var AllowChange: Boolean);
var
  Mask: TControlPxMask;
begin
{*** Because changing happens BEFORE PxMask.Validation, you have to validate manually,
     to prevent 'Cannot focus a disabled or invisible window' error. I hope i can
     fix this internally}
  if (GetOrdProperty(ActiveControl, 'PxMask', tkClass, LongInt(Mask)) and
    Mask.EditModified) then
  try
    ActiveControl := TPageControl(Sender);
    Mask.ValidateEdit;
  except
    AllowChange := false;
    Application.HandleException(Sender);
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  Mask: TPxMask;
begin
  if GetOrdProperty(LastMask, 'PxMask', tkClass, LongInt(Mask)) then
    with TdlgPxMaskEditor.create(self) do
    try
      PxMask := Mask;
      if ShowModal = idOK then
      begin
        Mask.assign(PxMask);
        SetPxMaskValues(Mask);
      end;
    finally
      Free;
    end;
end;

procedure TForm1.ActiveControlChanged(Sender: TObject);
var
  Mask: TControlPxMask;
begin
  if not (csDestroying in ComponentState) then
    if GetOrdProperty(ActiveControl, 'PxMask', tkClass, LongInt(Mask)) then
    begin
      SetPxMaskValues(Mask);
      LastMask := ActiveControl;
    end
    else if not (ActiveControl is TButton) then
      SetPxMaskValues(nil);
end;

procedure TForm1.DBPxGrid1ColEnter(Sender: TObject);
var
  Mask: TPxMask;
begin
  HasPxMask(DBPxGrid1.SelectedField, Mask);
  SetPxMaskValues(Mask);
end;

procedure TForm1.SetPxMaskValues(Mask: TPxMask);
begin
  if assigned(Mask) then
    with Mask do
    begin
      cbxAutoFill.checked := AutoFill;
      cbxRaiseInputError.checked := RaiseInputError;
      cbxAutoValidate.checked := AutoValidate;
      lblPxMask.Text := Mask;
      lblErrMsg.Text := ErrorMessage;
    end
  else
  begin
    cbxAutoFill.State := cbGrayed;
    cbxAutoFill.checked := false;
    cbxRaiseInputError.State := cbGrayed;
    cbxRaiseInputError.checked := false;
    cbxAutoValidate.State := cbGrayed;
    cbxAutoValidate.checked := false;
    lblPxMask.Text := '';
    lblErrMsg.Text := '';
  end;
  Button2.Enabled := assigned(Mask);
end;

procedure TForm1.PxTable1AfterOpen(DataSet: TDataSet);
begin
  UpdateDataset(TDBDataset(DataSet)); //Load values from DataDictionary
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  PxTable1.active := true;
end;

procedure TForm1.TabSheet1DragDrop(Sender, Source: TObject; X, Y: Integer);
begin

end;

end.
