unit dnPxProp;

(*****************************************************************************
Copyright  1997-99 Dieter Nth
version    : 2.1
date       : 10.08.1999
description:

  Editor for Paradox-style inputmasks

  1.03 Added:
    Example mask handling

  2.00 Added:
    ErrorMessage-Edit
******************************************************************************)

{$I dnInc.INC}

interface

uses
  Windows, SysUtils, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, Mask,
  dnGlobal, dnPxMask;

type
  TdlgPxMaskEditor = class(TForm)
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    Bevel1: TBevel;
    Label1: TLabel;
    edtMask: TEdit;
    btnRestore: TSpeedButton;
    Label2: TLabel;
    edtTest: TPxMaskEdit;
    GroupBox1: TGroupBox;
    cbMasks: TComboBox;
    btnLoad: TBitBtn;
    btnDelete: TBitBtn;
    btnAdd: TBitBtn;
    btnUse: TBitBtn;
    chkAutoFill: TCheckBox;
    chkAutoValidate: TCheckBox;
    chkRaiseInputError: TCheckBox;
    OpenDialog1: TOpenDialog;
    edtErrorMsg: TEdit;
    Label3: TLabel;
    Panel1: TPanel;
    Memo: TMemo;
    procedure GroupBox1Enter(Sender: TObject);
    procedure GroupBox1Exit(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnMasksClick(Sender: TObject);
    procedure cbMasksDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure cbMasksChange(Sender: TObject);
    procedure cbMasksKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure edtMaskChange(Sender: TObject);
    procedure edtTestChange(Sender: TObject);
    procedure btnUseClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnRestoreClick(Sender: TObject);
    procedure chkAutoFillClick(Sender: TObject);
    procedure chkAutoValidateClick(Sender: TObject);
    procedure chkRaiseInputErrorClick(Sender: TObject);
    procedure edtErrorMsgChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure edtTestValidChanged(Sender: TControlPxMask;
      EditControl: TWinControl; TextValid: Boolean);
  private
    { Private declarations }
    FPxMask: TPxMask;
    FOldHandler: TExceptionEvent;
    OldBeepOnPxError,
      OldBeepOnPxInputError,
      ListModified: Boolean;
    procedure SetPxMask(value: TPxMask);
    function GetPxMask: TPxMask;
    function GetListMaskValue(Index: Integer): string;
    function GetMaskValue(Index: Integer): string;
    procedure LoadMaskList(const FileName: string);
    procedure AppEx(Sender: TObject; E: Exception);
  protected
    procedure Loaded; override;
  public
    constructor create(aOwner: TComponent); override;
    destructor Destroy; override;
    property PxMask: TPxMask read GetPxMask write SetPxMask;
  end;

var
  dlgPxMaskEditor: TdlgPxMaskEditor;

function NameOf(const Param: string): string;
function ValueOf(const Param: string): string;

implementation

uses
  Registry,
  dnUtil, dnPxCnst;

{$R *.DFM}

const
{$IFDEF VER100}
  RegKey = 'Software\Borland\Delphi\3.0\';
{$ENDIF}
{$IFDEF VER120}
  RegKey = 'Software\Borland\Delphi\4.0\';
{$ENDIF}

function NameOf(const Param: string): string;
var
  P: Integer;
begin
  P := Pos('=', Param);
  if P > 1 then
    Result := copy(Param, 1, P - 1)
  else
    Result := '';
end;

function ValueOf(const Param: string): string;
var
  P: Integer;
begin
  P := Pos('=', Param);
  if P < length(Param) then
    Result := copy(Param, P + 1, MaxInt)
  else
    Result := '';
end;

{ TdlgPdoxMaskEditor }

constructor TdlgPxMaskEditor.create(aOwner: TComponent);
begin
  inherited create(aOwner);
  FPxMask := TPxMask.create(self);
end;

destructor TdlgPxMaskEditor.Destroy;
begin
  FPxMask.free;
  inherited Destroy;
end;

procedure TdlgPxMaskEditor.edtTestChange(Sender: TObject);
var
  s: string;
  i: Integer;
begin
  s := '';
  with edtTest do
    if length(text) <> 0 then
    begin
      i := SelStart;
      s := text;
      try
        PxMask.Validate(s);
        s := SPxPropEdInputOK;
      except
        on E: Exception do
        begin
          SelStart := i;
          s := text;
          if SelStart < PxMaskCheckInput(PxMask.Mask, s, chkAutoFill.checked)
            then
            s := SPxPropEdInputOkToCursor
          else
            s := E.Message;
        end;
      end;
    end;
  Memo.text := s;
end;

procedure TdlgPxMaskEditor.edtMaskChange(Sender: TObject);
begin
  btnRestore.enabled := true;
  edtTest.enabled := false;
  edtTest.Color := clBtnFace;
  try
    edtTest.PxMask.Mask := edtMask.text;
    if length(edtMask.text) > 0 then
    begin
      if edtTest.PxMask.Mask <> TrimRight(edtMask.text) then
        Memo.text := Format(SPxPropEdCommasRemoved,
          [edtTest.PxMask.Mask])
      else
        Memo.text := SPxPropEdSyntaxOK;
      edtTest.enabled := true;
      edtTest.Color := clWindow;
      btnAdd.enabled := true;
    end
    else
    begin
      Memo.text := '';
      btnAdd.enabled := false;
    end;
    btnOK.enabled := true;
  except
    on E: EPxMaskError do
    begin
      Memo.text := E.Message;
      btnOK.enabled := false;
      btnAdd.enabled := false;
    end;
  end;
end;

procedure TdlgPxMaskEditor.btnRestoreClick(Sender: TObject);
begin
  edtMask.text := FPxMask.Mask;
  btnRestore.enabled := false;
end;

function ReadMaskFileName: string;
var
  AppIniFile: TRegIniFile;
begin
  AppIniFile := TRegIniFile.create(RegKey);
  try
    Result := AppIniFile.ReadString(SIniEditorsName, SIniEditMaskName, '');
  finally
    AppIniFile.free;
  end;
end;

procedure TdlgPxMaskEditor.btnMasksClick(Sender: TObject);
var
  AppIniFile: TRegIniFile;
begin
  with OpenDialog1 do
    if Execute then
    begin
      LoadMaskList(FileName);
      AppIniFile := TRegIniFile.create(RegKey);
      try
        AppIniFile.WriteString(SIniEditorsName, SIniEditMaskName, FileName);
      finally
        AppIniFile.free;
      end;
    end;
end;

procedure TdlgPxMaskEditor.Loaded;
begin
  inherited Loaded;
  LoadMaskList(ReadMaskFileName);
end;

procedure TdlgPxMaskEditor.LoadMaskList(const FileName: string);
var
  i: Integer;
  Items: TStringList;
begin
  if (length(FileName) > 0) and FileExists(FileName) then
  begin
    Items := TStringList.create;
    try
      Items.LoadFromFile(FileName);
      i := 0;
      while i < (Items.Count - 1) do
      begin
        if (length(Items[i]) = 0) then
          Items.Delete(i)
        else if (Pos('=', Items[i]) = 0) then
          raise EPxMaskError.create(SInvalidSampleMask)
        else
          Inc(i);
        if i >= Items.Count then break;
      end;
      cbMasks.Items := Items;
      cbMasks.ItemIndex := 0;
      cbMasksChange(cbMasks);
    finally
      Items.free;
    end;
  end;
end;

procedure TdlgPxMaskEditor.cbMasksDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  with cbMasks do
  begin
    Canvas.FillRect(Rect);
    Canvas.TextOut(Rect.Left + 2, Rect.Top, GetMaskValue(Index));
  end;
end;

procedure TdlgPxMaskEditor.cbMasksChange(Sender: TObject);
begin
  Memo.text := GetListMaskValue(cbMasks.ItemIndex);
  btnDelete.enabled := (ActiveControl = cbMasks) and (cbMasks.ItemIndex >= 0);
  btnUse.enabled := GetMaskValue(cbMasks.ItemIndex) <> '';
end;

function TdlgPxMaskEditor.GetListMaskValue(Index: Integer): string;
begin
  if Index >= 0 then
    Result := NameOf(cbMasks.Items[Index])
  else
    Result := '?';
end;

function TdlgPxMaskEditor.GetMaskValue(Index: Integer): string;
begin
  Result := ValueOf(cbMasks.Items[Index]);
end;

procedure TdlgPxMaskEditor.btnUseClick(Sender: TObject);
begin
  edtMask.text := GetMaskValue(cbMasks.ItemIndex);
  edtMask.SetFocus;
  edtTest.text := '';
  btnAdd.enabled := false;
end;

procedure TdlgPxMaskEditor.cbMasksKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Word(Key) of
    VK_RETURN:
      begin
        if cbMasks.DroppedDown then btnUseClick(btnUse);
        cbMasks.DroppedDown := false;
      end;
    VK_ESCAPE:
      cbMasks.DroppedDown := false;
  end;
end;

procedure TdlgPxMaskEditor.btnDeleteClick(Sender: TObject);
var
  i: Integer;
begin
  with cbMasks do
    if (ItemIndex > 0) and
      (MessageDlg(Format(SPxPropEdConfirmDelete,
      [GetListMaskValue(cbMasks.ItemIndex)]),
      mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
    begin
      i := ItemIndex;
      Items.Delete(ItemIndex);
      if i >= Items.Count then i := Items.Count - 1;
      ItemIndex := i;
      cbMasksChange(self);
    end;
end;

procedure TdlgPxMaskEditor.btnAddClick(Sender: TObject);
var
  s: string;

  procedure SaveMask;
  begin
    cbMasks.Items.Values[s] := edtMask.text;
    ListModified := true;
  end;
begin
  s := '';
  while InputQuery(Format(SPxPropEdStore, [edtMask.text]), '', s) do
    if (s = '') then
    begin
      if (MessageDlg(SPxPropEdNeedDesc,
        mtError, [mbOK, mbCancel], 0) <> mrOK) then break;
    end
    else
    begin
      if (cbMasks.Items.Values[s] <> '') then
        case MessageDlg(Format(SPxPropEdOverwrite,
          [s]),
          mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
          mrYes: ;
          mrNo: continue;
        else
          break;
        end;
      cbMasks.Items.Values[s] := edtTest.PxMask.Mask;
      ListModified := true;
      break;
    end;
end;

procedure TdlgPxMaskEditor.GroupBox1Enter(Sender: TObject);
begin
  btnDelete.enabled := (cbMasks.ItemIndex >= 0);
end;

procedure TdlgPxMaskEditor.GroupBox1Exit(Sender: TObject);
begin
  btnDelete.enabled := false;
end;

procedure TdlgPxMaskEditor.FormDestroy(Sender: TObject);
begin
  if ListModified then
    cbMasks.Items.SaveToFile(ReadMaskFileName);
end;

function TdlgPxMaskEditor.GetPxMask: TPxMask;
begin
  FPxMask.Mask := edtTest.PxMask.Mask;
  FPxMask.AutoFill := chkAutoFill.checked;
  FPxMask.AutoValidate := chkAutoValidate.checked;
  FPxMask.RaiseInputError := chkRaiseInputError.checked;
  FPxMask.ErrorMessage := edtTest.PxMask.ErrorMessage;
  Result := FPxMask;
end;

procedure TdlgPxMaskEditor.SetPxMask(value: TPxMask);
begin
  FPxMask.assign(value);
  FPxMask.AutoValidate := false;
  FPxMask.RaiseInputError := false;

  edtTest.PxMask.Mask := value.Mask;
  edtTest.PxMask.ErrorMessage := value.ErrorMessage;

  edtMask.text := value.Mask;
  edtErrorMsg.text := value.ErrorMessage;
  edtTest.text := '';

  btnRestore.enabled := false;
  chkAutoFill.checked := value.AutoFill;
  chkAutoValidate.checked := value.AutoValidate;
  chkRaiseInputError.checked := value.RaiseInputError;
end;

procedure TdlgPxMaskEditor.chkAutoFillClick(Sender: TObject);
begin
  edtTest.PxMask.AutoFill := chkAutoFill.checked;
  FPxMask.AutoFill := chkAutoFill.checked;
end;

procedure TdlgPxMaskEditor.chkAutoValidateClick(Sender: TObject);
begin
  FPxMask.AutoValidate := chkAutoValidate.checked;
end;

procedure TdlgPxMaskEditor.chkRaiseInputErrorClick(Sender: TObject);
begin
  FPxMask.RaiseInputError := chkRaiseInputError.checked;
end;

procedure TdlgPxMaskEditor.edtErrorMsgChange(Sender: TObject);
begin
  edtTest.PxMask.ErrorMessage := edtErrorMsg.text;
end;

procedure TdlgPxMaskEditor.FormActivate(Sender: TObject);
begin
  OldBeepOnPxError := BeepOnPxError;
  OldBeepOnPxInputError := BeepOnPxInputError;
  BeepOnPxError := false;
  BeepOnPxInputError := false;
  edtTest.PxMask.AutoValidate := false;
  edtTest.PxMask.RaiseInputError := true;
  FOldHandler := Application.OnException;
  Application.OnException := AppEx;
end;

procedure TdlgPxMaskEditor.FormDeactivate(Sender: TObject);
begin
  BeepOnPxError := OldBeepOnPxError;
  BeepOnPxInputError := OldBeepOnPxInputError;
  Application.OnException := FOldHandler;
end;

procedure TdlgPxMaskEditor.AppEx(Sender: TObject; E: Exception);
begin
  if E is EPxMaskError then
    Memo.text := E.Message
  else
    Application.ShowException(E);
end;

procedure TdlgPxMaskEditor.edtTestValidChanged(Sender: TControlPxMask;
  EditControl: TWinControl; TextValid: Boolean);
begin
  if TextValid then
    TdlgPxMaskEditor(EditControl).Font.Color := clBlack
  else
    TdlgPxMaskEditor(EditControl).Font.Color := clRed;
end;

end.

