unit dnPxReg;

(*****************************************************************************
Copyright  1997-99 Dieter Nth
version    : 2.1
date       : 10.08.1999
description:

  Registers all Paradox-mask enabled components
******************************************************************************)

interface

{$I dnInc.INC}

procedure Register;

implementation

{$IFDEF WIN32}
{$R *.D32}
{$ELSE}
{$R *.D16}
{$ENDIF}

uses
  WinTypes, SysUtils, Classes, DB, Forms, Dialogs, Controls, DsgnIntf, DBTables,
  Grids,
  dnPxFlds, dnGlobal, dnPxMask, dnPxProp, dnDBPxM, dnPxTbl, dnPxGrid, dnDBPxG, dnDBUtil;


type
{*** TPdoxMaskProperty ***}
  TPxMaskProperty = class(TClassProperty)
  public
    function  GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

function TPxMaskProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paSubProperties, paDialog, paReadOnly];
end;

procedure TPxMaskProperty.Edit;
begin
  with TdlgPxMaskEditor.Create(application) do
  try
    PxMask := TPxMask(GetOrdValue);
    if ShowModal = idOK then
      SetOrdValue(LongInt(PxMask));
  finally
    Free;
  end;
end;

{*** TPxDataSetEditor ***}
{This is an example how to "inherit" from a ComponentEditor defined in the
 implementation section of another unit.
 Only NEW menu entries (Load/Store PxMasks) are handled, default entries
 (Fields editor / Explore) are passed to an instance of the "parent" class}
var
  PrevDataSetEditor: TComponentEditorClass;

type
  TPxDataSetEditor = class(TComponentEditor)
  private
    FPrevEditor: TComponentEditor;
  protected
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  public
    constructor Create(AComponent: TComponent; ADesigner:
        {$IFDEF VCL4+} IFormDesigner {$ELSE } TFormDesigner {$ENDIF}); override;
    destructor destroy; override;
  end;

constructor TPxDataSetEditor.Create(AComponent: TComponent; ADesigner:
{$IFDEF VCL4+} IFormDesigner {$ELSE } TFormDesigner {$ENDIF});
begin
  inherited;
  FPrevEditor := PrevDataSetEditor.create(AComponent, ADesigner);
end;

destructor TPxDataSetEditor.destroy;
begin
  inherited;
  FPrevEditor.free;
end;

procedure TPxDataSetEditor.ExecuteVerb(Index: Integer);
  procedure ShowSaveDialog(DataSet: TDataSet);
  begin
    with TSaveDialog.create(application) do try
      DefaultExt := 'pxm';
      Filter := 'Paradox masks (*.PXM)|*.PXM|All Files (*.*)|*.*';
      Title := 'Save Paradox Masks';
      if DataSet is TTable then
        FileName := QualifiedTableName(TTable(DataSet))
      else
        FileName := DataSet.Name;
      FileName := ChangeFileExt(FileName, PxMaskFileExt);
      Options := [ofOverwritePrompt, ofPathMustExist];
      if Execute then
        StorePxMasksToIni(DataSet, FileName, false); {Dialogs should be extended
       to contain a CheckBox to support this ^^^^^ ClearStoredFlag}
    finally
      free;
    end;
  end;

  procedure ShowOpenDialog(DataSet: TDataSet);
  begin
    with TOpenDialog.create(application) do try
      DefaultExt := 'pxm';
      Filter := 'Paradox masks (*.PXM)|*.PXM|All Files (*.*)|*.*';
      Title := 'Load Paradox Masks';
      if DataSet is TTable then
        FileName := QualifiedTableName(TTable(DataSet))
      else
        FileName := DataSet.Name;
      FileName := ChangeFileExt(FileName, PxMaskFileExt);
      Options := [ofHideReadOnly, ofPathMustExist, ofPathMustExist];
      if Execute then
        StorePxMasksToIni(DataSet, FileName, false); {Dialogs should be extended
       to contain a CheckBox to support this ^^^^^ ClearStoredFlag}
    finally
      free;
    end;
  end;

begin
  if index = FPrevEditor.GetVerbCount then
    ShowSaveDialog(TDataSet(Component))
  else
    if index = FPrevEditor.GetVerbCount + 1 then
      ShowOpenDialog(TDataSet(Component))
    else
      FPrevEditor.ExecuteVerb(Index);
end;

function TPxDataSetEditor.GetVerb(Index: Integer): string;
begin
  if index = FPrevEditor.GetVerbCount then
    Result := '&Save Pxmasks...'
  else
    if index = FPrevEditor.GetVerbCount + 1 then
      Result := '&Load Pxmasks...'
    else
      Result := FPrevEditor.GetVerb(Index);
end;

function TPxDataSetEditor.GetVerbCount: Integer;
begin
  Result := FPrevEditor.GetVerbCount;
  if (TDataSet(Component).FieldCount) > 0 then
    inc(Result, 2);
end;

{ *** Register *** }
procedure Register;
begin
  RegisterFields([ TPxStringField, TPxDateTimeField, TPxDateField, TPxTimeField,
                   TPxIntegerField, TPxSmallintField, TPxWordField, TPxFloatField,
                   TPxCurrencyField, TPxBCDField]);
  RegisterComponents('PxMask', [ TPxMaskEdit, TPxComboBox, TPxGrid, TDBPxEdit,
                                 TDBPxComboBox, TDBPxGrid, TPxTable]);

  RegisterPropertyEditor(TypeInfo(TPxMask), nil, 'PxMask', TPxMaskProperty);
{$IFDEF UseDD}
  {$IFDEF VCL3+} //Hide published properties in Object Inspector by undefining the property editor
  RegisterPropertyEditor(TypeInfo(string), TField, 'Px_Mask', nil);
  RegisterPropertyEditor(TypeInfo(string), TField, 'Px_ErrorMessage', nil);
  RegisterPropertyEditor(TypeInfo(boolean), TField, 'Px_AutoFill', nil);
  RegisterPropertyEditor(TypeInfo(boolean), TField, 'Px_AutoValidate', nil);
  RegisterPropertyEditor(TypeInfo(boolean), TField, 'Px_RaiseInputError', nil);
  RegisterPropertyEditor(TypeInfo(TPxMask), TPxGrid, 'PxMask', nil);
  RegisterPropertyEditor(TypeInfo(TPxMask), TDBPxGrid, 'PxMask', nil);
  {$ENDIF}
{$ENDIF}

{$IFDEF VCL3+}
  RegisterComponentEditor(TDBDataSet, TPxDataSetEditor);
{$ELSE}
  RegisterComponentEditor(TDataSet, TPxDataSetEditor);
{$ENDIF}
end;

{*** Set variable PrevDataSetEditor to the default editor-class for a DataSet ***}
procedure FindEditor;
var
  ds : TDataSet;
begin
{$IFDEF VCL3+}
  ds := TDBDataSet.create(nil);
{$ELSE}
  ds := TDataSet.create(nil);
{$ENDIF}
  try
    with GetComponentEditor(ds, nil) do
    begin
      TClass(PrevDataSetEditor):= ClassType;
      free;
    end;
  finally
    ds.free;
  end;
end;

initialization
  FindEditor;
end.
